@@ -940,61 +940,80 @@ update_font_width(AppletInfo * rss_appl_inf)
     pango_font_description_free(font2);
 }
 
-
-gchar *
-put_newline(gchar * title)
+/* calculates which byte the text is needed to break at at wrapping */
+gint
+calc_breaking_position(AppletInfo *rss_appl_inf, gchar *text)
 {
+    PangoContext *context;
+    PangoLayout *layout;
+    gint offs;
+    gint trailing;
+    gint width;
+    gchar *brk;
+
     ULOG_DEBUG(__FUNCTION__);
 
-    gint i = rss_appl_inf->chars_per_line - 1,
-         currentpos = 0;
-    gchar *string_return = NULL;
+    width = (gtk_tree_view_column_get_width(rss_appl_inf->textColumn) >
+                    HEADLINE_MIN_WIDTH)
+            ? gtk_tree_view_column_get_width(rss_appl_inf->textColumn)
+            : HEADLINE_MIN_WIDTH;
+    context = gtk_widget_get_pango_context(rss_appl_inf->rss_news);
+    layout = pango_layout_new(context);
+    pango_layout_set_markup(layout, text, -1);
+    ULOG_DEBUG("pango x: %d", width - rss_appl_inf->avg_char_width);
+    pango_layout_xy_to_index(layout, 
+            (width - 2*rss_appl_inf->avg_char_width) * PANGO_SCALE, 
+            0, &offs, &trailing);
+    g_object_unref(layout);
+    ULOG_DEBUG("xy index %d trailing %d", offs, trailing);
+    brk = g_utf8_offset_to_pointer(text + offs, trailing);
+    ULOG_DEBUG("byte offset: %d", brk - text);
+    return brk - text;
+}
 
+/* puts a newline at a space before offset (counted in bytes) or 
+ * at offset it there's no space before that */
+gchar *
+put_newline(gchar *title, gint offset)
+{
+    gint i;
+
+    ULOG_DEBUG(__FUNCTION__);
 
     if (NULL == title)
     {
         return NULL;
     }
-
-    ULOG_DEBUG("The string: %s", title);
-
-    if (strlen(title) > rss_appl_inf->chars_per_line)
-    {
-        /* We have to search the first space, looking backward */
-        while (i && (title[i] != ' '))
-        {
-            i--;
-        }
-
-        if (i > 0)
-        {
-            /* found space, breaking the line there */
-            title[i] = '\n';
-            currentpos = i + 1;
-        }
-        else
-        {
-            /* there was no space inside. we wrap the text simply */
-            gchar *temp = NULL;
-
-            temp = g_strdup(title + rss_appl_inf->chars_per_line - 1);
-            title[rss_appl_inf->chars_per_line-1] = '\n';
-            title[rss_appl_inf->chars_per_line] = '\0';
-            string_return = g_strjoin(NULL, title, temp, NULL);
-
-            g_free(temp);
-            g_free(title);
-
-            title = string_return;
-        }
-    }
-    else
+    
+    if (offset >= strlen(title))
     {
         return title;
     }
+
+    i = offset;
+    while (i > 0 && title[i] != ' ')
+        --i;
+    
+    if (i > 0)
+    {
+        title[i] = '\n';
+        return title;
+    }
+
+    /* there was no space inside. we wrap the text simply */
+    gchar *part1 = title;
+    gchar *part2 = g_strdup(title + offset);
+    part1[offset] = '\n';
+    part1[offset+1] = 0;
+    title = g_strjoin(NULL, part1, part2, NULL);
+    g_free(part1);
+    g_free(part2);
+
     return title;
 }
 
+
+
 /** Creates an empty file
   *
   * @param file the filename to create
@@ -1563,7 +1582,7 @@ create_headline_display(AppletInfo * rss_appl_inf)
     gtk_tree_view_append_column(GTK_TREE_VIEW(rss_news), column);
     gtk_tree_view_column_pack_start(column, textRenderer, TRUE);
     gtk_tree_view_column_add_attribute(column, textRenderer, "markup",
-                                       RENDER_COLUMN);
+                    RENDER_COLUMN);
     g_object_set(textRenderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
     GtkTreeSelection *rss_select =
@@ -2063,13 +2082,16 @@ _applet_set_headlines(AppletInfo * applet)
             num++;
             if (num <= MAX_HEADLINES)
             {
-                /* Processing headline information */
+               /* Processing headline information */
                 if (headline->title)
                 {
                     item_title = g_markup_escape_text(headline->title, -1);
                     item_title = g_strstrip(item_title);
                     item_title = g_strdelimit(item_title, "\r\n\t", ' ');
-                    item_title = put_newline(item_title);
+                    gint brk = calc_breaking_position(applet, item_title);
+                    ULOG_DEBUG("The string: %s", item_title);
+                    item_title = put_newline(item_title, brk);
+                    ULOG_DEBUG("wrapped as:\n>%s<", item_title);
                 }
                 else
                 {
@@ -2082,6 +2104,7 @@ _applet_set_headlines(AppletInfo * applet)
                     pixbuf = headline->icon;
 
                 /* Adding the title */
+                ULOG_DEBUG("adding title %d %s", num, item_title);
 
                 /* If displayed_headlines <= MAX_HEADLINES, animation needed. 
                  * Appending first few items to end of list, to enable smooth
